//
// Copyright (c) 1999-2000 Tony Givargis.  Permission to copy is granted
// provided that this header remains intact.  This software is provided
// with no warranties.
//
// Version : 2.8
//

//////////////////////////////////////////////////////////////////////////////-

//////////////////////////////////////////////////////////////////////////////-

//
// rst (active hi) : resets the 8051
// clk (rising edge) : clocks the 8051
// xrm_addr : this is the address of xram being requested
// xrm_out_data : this is the data sent to the xram
// xrm_in_data : this is the data received from the xram
// xrm_rd (active lo) : asserted to wire a xram read
// xrm_wr (active lo) : asserted to wire a xram write
// p0_in : port 0's input direction
// p0_out : port 0's output direction
// p0_in : port 1's input direction
// p0_out : port 1's output direction
// p0_in : port 2's input direction
// p0_out : port 2's output direction
// p0_in : port 3's input direction
// p0_out : port 3's output direction
//
`include "time_scale.v"
`include "i8051_lib.v"
module I8051_ALL (
    rst          ,
    clk          ,
    xrm_addr     ,
    xrm_out_data ,
    xrm_in_data  ,
    xrm_rd       ,
    xrm_wr       ,
    p0_in        ,
    p0_out       ,
    p1_in        ,
    p1_out       ,
    p2_in        ,
    p2_out       ,
    p3_in        ,
    p3_out        
  );
  
  input          rst;
  input          clk;
  output [15:0]  xrm_addr;
  output [ 7:0]  xrm_out_data;
  input  [ 7:0]  xrm_in_data;
  output         xrm_rd;
  output         xrm_wr;
  input  [ 7:0]  p0_in;
  output [ 7:0]  p0_out;
  input  [ 7:0]  p1_in;
  output [ 7:0]  p1_out;
  input  [ 7:0]  p2_in;
  output [ 7:0]  p2_out;
  input  [ 7:0]  p3_in;
  output [ 7:0]  p3_out;
  
  //////////////////////////////////////////////////////////////////////////////-
  
  
  wire [11:0] rom_addr          ;
  wire [ 7:0] rom_data          ;
  wire rom_rd                   ;
  wire [ 7:0] ram_addr          ;
  wire [ 7:0] ram_out_data      ;
  wire [ 7:0] ram_in_data       ;
  wire ram_out_bit_data         ;
  wire ram_in_bit_data          ;
  wire ram_rd                   ;
  wire ram_wr                   ;
  wire ram_is_bit_addr          ;
  wire [ 7:0] dec_op_out        ;
  wire [ 8:0] dec_op_in         ;
  wire [ 3:0] alu_op_code       ;
  wire [ 7:0] alu_src_1         ;
  wire [ 7:0] alu_src_2         ;
  wire [ 7:0] alu_src_3         ;
  wire alu_src_cy               ;
  wire alu_src_ac               ;
  wire [ 7:0] alu_des_1         ;
  wire [ 7:0] alu_des_2         ;
  wire alu_des_cy               ;
  wire alu_des_ac               ;
  wire alu_des_ov               ;
  
  //Pini
  wire dll                      ;
  
  //Pini
  //unisim
  CLKDLL u_dll (
  //simprim
  //X_CLKDLL u_dll (
    .CLK0(dll),
    //  .CLK180,
    //  .CLK270,
    //  .CLK2X,
    //  .CLK90,
    //  .CLKDV,
    //  .LOCKED(locked),
    
    //input
    .CLKFB(dll),
    .CLKIN(clk),
    .RST  (1'b0)
    //.RST  (rst )
  );//u_dll : CLKDLL
  
  I8051_ALU U_ALU (
  //.rst    (rst        ),
    .op_code(alu_op_code),
    .src_1 (alu_src_1 ),
    .src_2 (alu_src_2 ),
    .src_3 (alu_src_3 ),
    .src_cy(alu_src_cy),
    .src_ac(alu_src_ac),
    .des_1 (alu_des_1 ),
    .des_2 (alu_des_2 ),
    .des_cy(alu_des_cy),
    .des_ac(alu_des_ac),
    .des_ov(alu_des_ov)
  );//I8051_ALU U_ALU
  
  I8051_DEC U_DEC (
    .rst   (rst       ),
    .op_in (dec_op_out),
    .op_out(dec_op_in )
  );//I8051_DEC U_DEC
  
  I8051_RAM U_RAM (
    .rst(rst),
    .clk(dll),
    
    .addr        (ram_addr        ),
    .in_data     (ram_in_data     ),
    .out_data    (ram_out_data    ),
    .in_bit_data (ram_out_bit_data ),
    .out_bit_data(ram_in_bit_data),
    .rd          (ram_rd          ),
    .wr          (ram_wr          ),
    .is_bit_addr (ram_is_bit_addr ),
    .p0_in       (p0_in           ),
    .p0_out      (p0_out          ),
    .p1_in       (p1_in           ),
    .p1_out      (p1_out          ),
    .p2_in       (p2_in           ),
    .p2_out      (p2_out          ),
    .p3_in       (p3_in           ),
    .p3_out      (p3_out          )
  );//I8051_RAM U_RAM
  
  I8051_ROM U_ROM (
    .rst (rst     ),
    .clk (dll     ),
    .addr(rom_addr),
    .data(rom_data),
    .rd  (rom_rd  )
  );//I8051_ROM U_ROM
  
  I8051_CTR U_CTR (
    .rst(rst),
    .clk(dll),
    
    .rom_addr(rom_addr),
    .rom_data(rom_data),
    .rom_rd  (rom_rd  ),
    
    .ram_addr        (ram_addr        ),
    .ram_out_data    (ram_in_data     ),
    .ram_in_data     (ram_out_data    ),
    .ram_in_bit_data (ram_in_bit_data ),
    .ram_out_bit_data(ram_out_bit_data),
    .ram_rd          (ram_rd          ),
    .ram_wr          (ram_wr          ),
    .ram_is_bit_addr (ram_is_bit_addr ),
    
    .xrm_addr        (xrm_addr    ),
    .xrm_out_data    (xrm_out_data),
    .xrm_in_data     (xrm_in_data ),
    .xrm_rd          (xrm_rd      ),
    .xrm_wr          (xrm_wr      ),
    
    .dec_op_out(dec_op_out),
    .dec_op_in (dec_op_in ),
    
    .alu_op_code(alu_op_code),
    .alu_src_1  (alu_src_1 ),
    .alu_src_2  (alu_src_2 ),
    .alu_src_3  (alu_src_3 ),
    .alu_src_cy (alu_src_cy),
    .alu_src_ac (alu_src_ac),
    .alu_des_1  (alu_des_1 ),
    .alu_des_2  (alu_des_2 ),
    .alu_des_cy (alu_des_cy),
    .alu_des_ac (alu_des_ac),
    .alu_des_ov (alu_des_ov)
  );//I8051_CTR U_CTR
  
  `ifdef SIM_ONLY
  I8051_DBG U_DBG (
    .op_in(dec_op_in)
  );
  `endif
endmodule

//////////////////////////////////////////////////////////////////////////////-

// end of file //
