//
// Copyright (c) 1999-2000 Tony Givargis.  Permission to copy is granted
// provided that this header remains intact.  This software is provided
// with no warranties.
//
// Version : 2.8a
//

//////////////////////////////////////////////////////////////////////////////-

//
// rst (active hi) : alu outputs don't car values during reset
// op_code : defines the alu operation (see I8051_LIB)
// src_1 : first source operand of the alu
// src_2 : second source operand of the alu
// src_3 : third source operand of the alu
// src_cy : carry into the 7th bit of the alu
// src_ac : carry into the 4th bit of the alu
// des_1 : first destination operand of the alu
// des_2 : second destination operand of the alu
// des_cy : carry out of the 7th bit of the alu
// des_ac : carry out of the 4th bit of the alu
// des_ov : overflow out of the alu
//
`include "time_scale.v"
`include "i8051_lib.v"
module I8051_ALU (
    rst     ,
    op_code ,
    src_1   ,
    src_2   ,
    src_3   ,
    src_cy  ,
    src_ac  ,
    des_1   ,
    des_2   ,
    des_cy  ,
    des_ac  ,
    des_ov  
  );
  input          rst;
  input  [3:0]   op_code;
  input  [7:0]   src_1;
  input  [7:0]   src_2;
  input  [7:0]   src_3;
  input          src_cy;
  input          src_ac;
  output [7:0]   des_1;
  output [7:0]   des_2;
  output         des_cy;
  output         des_ac;
  output         des_ov;
  
  //////////////////////////////////////////////////////////////////////////////-
  
  reg [7:0]   des_1;
  reg [7:0]   des_2;
  reg         des_cy;
  reg         des_ac;
  reg         des_ov;
  
  //////////////////////////////////////////////////////////////////////////////-
  
  reg [15:0] v16;
  reg [ 7:0]  v8;
  reg v_cy, v_ac, v_ov;
  
  
  task PCSADD;
    input  [15:0]  a;
    input  [ 7:0]  b;
    output [15:0]  r;
    
    reg [15:0] v1, v2;
    reg [15:0] r;//+
    begin
      v1 = a;
      if( b[7] == 1'b1 ) 
      v2 = {`CM_8, b};
      else
      v2 = {`C0_8, b};
      v1 = v1 + v2;
      r  = v1;
    end 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task PCUADD;
    input  [15:0]  a;
    input  [ 7:0]  b;
    output [15:0]  r;
    reg    [15:0]  r;
    begin
      r = a + b;
    end 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task DO_ADD;
    input  [7:0] a, b;
    input           c;
    output [7:0]    r;
    output       cy, ac, ov;
    reg    [7:0]    r;
    reg          cy, ac, ov;
    
    reg  [4:0] v1, v2, v3, v4;
    reg  [3:0] v5, v6, v7, v8;
    reg  [1:0] v9, vA, vB, vC;
    begin
      
      v1 = {1'b0, a[3:0]};
      v2 = {1'b0, b[3:0]};
      v3 = {1'b0, 3'b000, c};
      v4 = v1 + v2 + v3;
      
      v5 = {1'b0, a[6:4]};
      v6 = {1'b0, b[6:4]};
      v7 = {1'b0, 2'b00, v4[4]};
      v8 = v5 + v6 + v7;
      
      v9 = {1'b0, a[7]};
      vA = {1'b0, b[7]};
      vB = {1'b0, v8[3]};
      vC = v9 + vA + vB;
      
      r[7] = vC[0];
      r[6] = v8[2];
      r[5] = v8[1];
      r[4] = v8[0];
      r[3] = v4[3];
      r[2] = v4[2];
      r[1] = v4[1];
      r[0] = v4[0];
      cy   = vC[1];
      ac   = v4[4];
      ov   = vC[1] ^ v8[3];
    end
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task DO_SUB;
    input  [7:0] a, b;
    input           c;
    output [7:0]    r;
    output       cy, ac, ov;
    reg    [7:0]    r;
    reg          cy, ac, ov;
    
    reg [4:0] v1, v2, v3, v4;
    reg [3:0] v5, v6, v7, v8;
    reg [1:0] v9, vA, vB, vC;
    begin
      
      v1 = {1'b1, a[3:0]};
      v2 = {1'b0, b[3:0]};
      v3 = {1'b0, 3'b000, c};
      v4 = v1 - v2 - v3;
      
      v5 = {1'b1, a[6:4]};
      v6 = {1'b0, b[6:4]};
      v7 = {1'b0, 2'b00, (~v4[4])};
      v8 = v5 - v6 - v7;
      
      v9 = {1'b1, a[7]};
      vA = {1'b0, b[7]};
      vB = {1'b0, (~v8[3])};
      vC = v9 - vA - vB;
      
      r[7] = vC[0];
      r[6] = v8[2];
      r[5] = v8[1];
      r[4] = v8[0];
      r[3] = v4[3];
      r[2] = v4[2];
      r[1] = v4[1];
      r[0] = v4[0];
      cy = ~vC[1];
      ac = ~v4[4];
      ov = (~vC[1]) ^ (~v8[3]);
    end
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task DO_MUL;
    input  [ 7:0] a, b;
    output [15:0]    r;
    output           ov;
    reg    [15:0]    r;
    reg              ov;
    
    reg [15:0] v1;
    begin
      v1 = a * b;
      r  = v1;
      if( v1[15:8] != `C0_8 ) 
      ov = 1'b1;
      else
      ov = 1'b0;
    end
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task DO_DIV;
    input  [ 7:0] a, b;
    output [15:0]    r;
    output          ov;
    reg    [15:0]    r;
    reg             ov;
    
    reg [15:0] v1     ;
    reg [ 8:0] v2, v3 ;
    integer i;
    begin
      if( b == `C0_8 ) begin
        r[ 7:0] = `CD_8;
        r[15:8] = `CD_8;
        ov = 1'b1;
      end
      else begin 
        if( a == b ) begin
          r[ 7:0] = `C1_8;
          r[15:8] = `C0_8;
          ov = 1'b0;
        end
        else begin
          if( a < b ) begin
            r[ 7:0] = `C0_8;
            r[15:8] = src_1;
            ov = 1'b0;
          end
          else begin
            v1[ 7:0] = a;
            v1[15:8] = `C0_8;
            v3       = {1'b0, b};
            for (i=0; i <= 7; i=i+1) begin
              v1[15:1] = v1[14:0];
              v1[0] = 1'b0;
              v2 = {1'b1, v1[15:8]};
              v2 = v2 - v3;
              if( v2[8] == 1'b1 ) begin
                v1[0] = 1'b1;
                v1[15:8] = v2[7:0];
              end
            end //for
            
            r  = v1;
            ov = 1'b0;
          end
        end
      end
    end
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task DO_DA;
    input  [7:0] a;
    input        c1, c2;
    output [7:0] r;
    output       cy;
    reg    [7:0] r;
    reg          cy;
    
    reg [8:0] v;
    begin
      v = {1'b0, a};
      if( (c2 == 1'b1) || (v[3:0] > `C9_4) ) begin
        v = v + 9'b0_0000_0110;
      end
      v[8] = v[8] | c1;
      if( v[8] == 1'b1 || (v[7:4] > `C9_4) ) begin
        v = v + 9'b0_0110_0000;
      end
      r  = v[7:0];
      cy = v[8];
    end
  endtask
  
  
  
  always @ (
    rst or op_code or src_1 or src_2 or src_3 or src_cy or src_ac
  ) begin
    if( rst == 1'b1 ) begin
      des_1  = `CD_8;
      des_2  = `CD_8;
      des_cy =  1'b0;
      des_ac =  1'b0;
      des_ov =  1'b0;
      v16    = 16'h0;
      v8     =  8'h0;
      v_cy   =  1'b0;
      v_ac   =  1'b0;
      v_ov   =  1'b0;
    end
    else begin
      v16    = 16'h0;
      v8     =  8'h0;
      v_cy   =  1'b0;
      v_ac   =  1'b0;
      v_ov   =  1'b0;
      case (op_code) // synthesis parallel_case full_case
        `ALU_OPC_ADD   :  begin
          DO_ADD(src_1, src_2, src_cy, v8, v_cy, v_ac, v_ov);
          des_1  = v8;
          des_2  = `CD_8;
          des_cy = v_cy;
          des_ac = v_ac;
          des_ov = v_ov;
        end
        `ALU_OPC_SUB   :  begin
          DO_SUB(src_1, src_2, src_cy, v8, v_cy, v_ac, v_ov);
          des_1  = v8;
          des_2  = `CD_8;
          des_cy = v_cy;
          des_ac = v_ac;
          des_ov = v_ov;
        end
        `ALU_OPC_MUL   :  begin
          //Pini
          //DO_MUL(src_1, src_2, v16, v_ov);
          //des_1  = v16[ 7:0];
          //des_2  = v16[15:8];
          //des_cy = 1'b0;
          //des_ac = 1'b0;
          //des_ov = v_ov;
          des_1  = 8'd0;
          des_2  = 8'd0;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_DIV   :  begin
          //Pini
          //DO_DIV(src_1, src_2, v16, v_ov);
          //des_1  = v16[ 7:0];
          //des_2  = v16[15:8];
          //des_cy = 1'b0;
          //des_ac = 1'b0;
          //des_ov = v_ov;
          des_1  = 8'd0;
          des_2  = 8'd0;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_DA    :  begin
          DO_DA(src_1, src_cy, src_ac, v8, v_cy);
          des_1  = v8;
          des_2  = `CD_8;
          des_cy = v_cy;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_NOT   :  begin
          des_1[7] = ~src_1[7];
          des_1[6] = ~src_1[6];
          des_1[5] = ~src_1[5];
          des_1[4] = ~src_1[4];
          des_1[3] = ~src_1[3];
          des_1[2] = ~src_1[2];
          des_1[1] = ~src_1[1];
          des_1[0] = ~src_1[0];
          des_2  = `CD_8;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_AND   :  begin
          des_1[7] = src_1[7] & src_2[7];
          des_1[6] = src_1[6] & src_2[6];
          des_1[5] = src_1[5] & src_2[5];
          des_1[4] = src_1[4] & src_2[4];
          des_1[3] = src_1[3] & src_2[3];
          des_1[2] = src_1[2] & src_2[2];
          des_1[1] = src_1[1] & src_2[1];
          des_1[0] = src_1[0] & src_2[0];
          des_2  = `CD_8;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_XOR   :  begin
          des_1[7] = src_1[7] ^ src_2[7];
          des_1[6] = src_1[6] ^ src_2[6];
          des_1[5] = src_1[5] ^ src_2[5];
          des_1[4] = src_1[4] ^ src_2[4];
          des_1[3] = src_1[3] ^ src_2[3];
          des_1[2] = src_1[2] ^ src_2[2];
          des_1[1] = src_1[1] ^ src_2[1];
          des_1[0] = src_1[0] ^ src_2[0];
          des_2  = `CD_8;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_OR    :  begin
          des_1[7] = src_1[7] | src_2[7];
          des_1[6] = src_1[6] | src_2[6];
          des_1[5] = src_1[5] | src_2[5];
          des_1[4] = src_1[4] | src_2[4];
          des_1[3] = src_1[3] | src_2[3];
          des_1[2] = src_1[2] | src_2[2];
          des_1[1] = src_1[1] | src_2[1];
          des_1[0] = src_1[0] | src_2[0];
          des_2  = `CD_8;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_RL    :  begin
          des_1[  0] = src_1[  7];
          des_1[7:1] = src_1[6:0];
          des_2  = `CD_8;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_RLC   :  begin
          des_1[  0] = src_cy;
          des_1[7:1] = src_1[6:0];
          des_2  = `CD_8;
          des_cy = src_1[7];
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_RR    :  begin
          des_1[  7] = src_1[  0];
          des_1[6:0] = src_1[7:1];
          des_2  = `CD_8;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_RRC   :  begin
          des_1[  7] = src_cy;
          des_1[6:0] = src_1[7:1];
          des_2  = `CD_8;
          des_cy = src_1[0];
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_PCSADD:  begin
          PCSADD({src_2, src_1}, src_3, v16);
          des_1  = v16[ 7:0];
          des_2  = v16[15:8];
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        `ALU_OPC_PCUADD:  begin
          PCUADD({src_2, src_1}, src_3, v16);
          des_1  = v16[ 7:0];
          des_2  = v16[15:8];
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
        default: begin
          des_1  = `CD_8;
          des_2  = `CD_8;
          des_cy = 1'b0;
          des_ac = 1'b0;
          des_ov = 1'b0;
        end
      endcase
    end
  end
endmodule

//////////////////////////////////////////////////////////////////////////////-

// end of file //


