//
// Copyright (c) 1999-2000 Tony Givargis.  Permission to copy is granted
// provided that this header remains intact.  This software is provided
// with no warranties.
//
// Version : 2.8a
//

//////////////////////////////////////////////////////////////////////////////-

//
// rst (active hi) : alu outputs don't car values during reset
// op_code : defines the alu operation (see I8051_LIB)
// src_1 : first source operand of the alu
// src_2 : second source operand of the alu
// src_3 : third source operand of the alu
// src_cy : carry into the 7th bit of the alu
// src_ac : carry into the 4th bit of the alu
// des_1 : first destination operand of the alu
// des_2 : second destination operand of the alu
// des_cy : carry out of the 7th bit of the alu
// des_ac : carry out of the 4th bit of the alu
// des_ov : overflow out of the alu
//
`include "time_scale.v"
`include "i8051_lib.v"
module I8051_ALU (
    clk     ,
    rst     ,
    op_code ,
    src_1   ,
    src_2   ,
    src_3   ,
    src_cy  ,
    src_ac  ,
    des_1   ,
    des_2   ,
    des_cy  ,
    des_ac  ,
    des_ov  
  );
  input          clk;
  input          rst;
  input  [3:0]   op_code;
  input  [7:0]   src_1;
  input  [7:0]   src_2;
  input  [7:0]   src_3;
  input          src_cy;
  input          src_ac;
  output [7:0]   des_1;
  output [7:0]   des_2;
  output         des_cy;
  output         des_ac;
  output         des_ov;

  reg    [7:0]   des_1;
  reg    [7:0]   des_2;
  reg            des_cy;
  reg            des_ac;
  reg            des_ov;
  wire   [7:0]   des_1_i;
  wire   [7:0]   des_2_i;
  wire           des_cy_i;
  wire           des_ac_i;
  wire           des_ov_i;


  wire [15:0] pcsadd;
  wire [15:0] pcuadd;
  wire [ 7:0] add_r;
  wire        add_cy, add_ac, add_ov;
  wire [ 7:0] sub_r;
  wire        sub_cy, sub_ac, sub_ov;
  wire [15:0] mul_r;
  wire        mul_ov;
  wire [15:0] div_r;
  wire        div_ov;
  wire [ 7:0] da_r;
  wire        da_cy;
  wire [ 7:0] des_1_n; //not
  wire [ 7:0] des_1_a; //and
  wire [ 7:0] des_1_x; //xor
  wire [ 7:0] des_1_r; // or
  wire [ 7:0] des_rl ; // rl
  wire [ 7:0] des_rlc; //rlc
  wire [ 7:0] des_rr ; // rr
  wire [ 7:0] des_rrc; //rrc
  
  ////////////////////////////////////
  PCSADD u_pcsadd (
    .a({src_2, src_1}),
    .b(src_3),
    .r(pcsadd)
  );//PCSADD u_pcsadd
  PCUADD u_pcuadd (
    .a({src_2, src_1}),
    .b(src_3),
    .r(pcuadd)
  );//PCUADD u_pcuadd
  DO_ADD u_add (
    .a (src_1 ),
    .b (src_2 ),
    .c (src_cy),
    .r (add_r ),
    .cy(add_cy),
    .ac(add_ac),
    .ov(add_ov)
  );//DO_ADD u_add
  DO_SUB u_sub (
    .a (src_1 ),
    .b (src_2 ),
    .c (src_cy),
    .r (sub_r ),
    .cy(sub_cy),
    .ac(sub_ac),
    .ov(sub_ov)
  );//DO_SUB u_sub

/*
  DO_MUL u_mul (
    .a (src_1 ),
    .b (src_2 ),
    .r (mul_r ),
    .ov(mul_ov)
  );//DO_MUL u_mul
  DO_DIV u_div (
    .a (src_1 ),
    .b (src_2 ),
    .r (div_r ),
    .ov(div_ov)
  );//DO_DIV u_div
*/
  DO_DA u_da (
    .a (src_1 ),
    .c1(src_cy),
    .c2(src_ac),
    .r (da_r  ),
    .cy(da_cy )
  );//DO_DA u_da

  //not
  assign des_1_n[7] = ~src_1[7];
  assign des_1_n[6] = ~src_1[6];
  assign des_1_n[5] = ~src_1[5];
  assign des_1_n[4] = ~src_1[4];
  assign des_1_n[3] = ~src_1[3];
  assign des_1_n[2] = ~src_1[2];
  assign des_1_n[1] = ~src_1[1];
  assign des_1_n[0] = ~src_1[0];
  //and
  assign des_1_a[7] = src_1[7] & src_2[7];
  assign des_1_a[6] = src_1[6] & src_2[6];
  assign des_1_a[5] = src_1[5] & src_2[5];
  assign des_1_a[4] = src_1[4] & src_2[4];
  assign des_1_a[3] = src_1[3] & src_2[3];
  assign des_1_a[2] = src_1[2] & src_2[2];
  assign des_1_a[1] = src_1[1] & src_2[1];
  assign des_1_a[0] = src_1[0] & src_2[0];
  //xor
  assign des_1_x[7] = src_1[7] ^ src_2[7];
  assign des_1_x[6] = src_1[6] ^ src_2[6];
  assign des_1_x[5] = src_1[5] ^ src_2[5];
  assign des_1_x[4] = src_1[4] ^ src_2[4];
  assign des_1_x[3] = src_1[3] ^ src_2[3];
  assign des_1_x[2] = src_1[2] ^ src_2[2];
  assign des_1_x[1] = src_1[1] ^ src_2[1];
  assign des_1_x[0] = src_1[0] ^ src_2[0];
  //or
  assign des_1_r[7] = src_1[7] | src_2[7];
  assign des_1_r[6] = src_1[6] | src_2[6];
  assign des_1_r[5] = src_1[5] | src_2[5];
  assign des_1_r[4] = src_1[4] | src_2[4];
  assign des_1_r[3] = src_1[3] | src_2[3];
  assign des_1_r[2] = src_1[2] | src_2[2];
  assign des_1_r[1] = src_1[1] | src_2[1];
  assign des_1_r[0] = src_1[0] | src_2[0];
  //rl
  assign des_rl[  0] = src_1[  7];
  assign des_rl[7:1] = src_1[6:0];
  //rlc
  assign des_rlc[  0] = src_cy;
  assign des_rlc[7:1] = src_1[6:0];
  //rr
  assign des_rr[  7] = src_1[  0];
  assign des_rr[6:0] = src_1[7:1];
  //rrc
  assign des_rrc[  7] = src_cy;
  assign des_rrc[6:0] = src_1[7:1];


assign des_1_i=
(op_code == `ALU_OPC_ADD   ) ? add_r :
(op_code == `ALU_OPC_SUB   ) ? sub_r :
//(op_code == `ALU_OPC_MUL   ) ? mul_r[7:0] :
//(op_code == `ALU_OPC_DIV   ) ? div_r[7:0] :
(op_code == `ALU_OPC_MUL   ) ? `CD_8 :
(op_code == `ALU_OPC_DIV   ) ? `CD_8 :
(op_code == `ALU_OPC_DA    ) ? da_r  :
(op_code == `ALU_OPC_NOT   ) ? des_1_n :
(op_code == `ALU_OPC_AND   ) ? des_1_a :
(op_code == `ALU_OPC_XOR   ) ? des_1_x :
(op_code == `ALU_OPC_OR    ) ? des_1_r : 
(op_code == `ALU_OPC_RL    ) ? des_rl  :
(op_code == `ALU_OPC_RLC   ) ? des_rlc :
(op_code == `ALU_OPC_RR    ) ? des_rr  :
(op_code == `ALU_OPC_RRC   ) ? des_rrc :
(op_code == `ALU_OPC_PCSADD) ? pcsadd[7:0] :
(op_code == `ALU_OPC_PCUADD) ? pcuadd[7:0] : `CD_8;
//
assign des_2_i=
(op_code == `ALU_OPC_ADD   ) ? `CD_8 :
(op_code == `ALU_OPC_SUB   ) ? `CD_8 :
//(op_code == `ALU_OPC_MUL   ) ? mul_r[15:8] :
//(op_code == `ALU_OPC_DIV   ) ? div_r[15:8] :
(op_code == `ALU_OPC_MUL   ) ? `CD_8 :
(op_code == `ALU_OPC_DIV   ) ? `CD_8 :
(op_code == `ALU_OPC_DA    ) ? `CD_8 :
(op_code == `ALU_OPC_NOT   ) ? `CD_8 :
(op_code == `ALU_OPC_AND   ) ? `CD_8 :
(op_code == `ALU_OPC_XOR   ) ? `CD_8 :
(op_code == `ALU_OPC_OR    ) ? `CD_8 : 
(op_code == `ALU_OPC_RL    ) ? `CD_8  :
(op_code == `ALU_OPC_RLC   ) ? `CD_8 :
(op_code == `ALU_OPC_RR    ) ? `CD_8  :
(op_code == `ALU_OPC_RRC   ) ? `CD_8 :
(op_code == `ALU_OPC_PCSADD) ? pcsadd[15:8] :
(op_code == `ALU_OPC_PCUADD) ? pcuadd[15:8] : `CD_8;
//
assign des_cy_i=
(op_code == `ALU_OPC_ADD   ) ? add_cy  :
(op_code == `ALU_OPC_SUB   ) ? sub_cy  :
(op_code == `ALU_OPC_MUL   ) ? 1'b0    :
(op_code == `ALU_OPC_DIV   ) ? 1'b0    :
(op_code == `ALU_OPC_DA    ) ? da_cy   :
(op_code == `ALU_OPC_NOT   ) ? 1'b0    :
(op_code == `ALU_OPC_AND   ) ? 1'b0    :
(op_code == `ALU_OPC_XOR   ) ? 1'b0    :
(op_code == `ALU_OPC_OR    ) ? 1'b0    :
(op_code == `ALU_OPC_RL    ) ? 1'b0    :
(op_code == `ALU_OPC_RLC   ) ? src_1[7]:
(op_code == `ALU_OPC_RR    ) ? 1'b0    :
(op_code == `ALU_OPC_RRC   ) ? src_1[0]:
(op_code == `ALU_OPC_PCSADD) ? 1'b0    :
(op_code == `ALU_OPC_PCUADD) ? 1'b0    : 1'b0;
//
assign des_ac_i=
(op_code == `ALU_OPC_ADD   ) ? add_ac  :
(op_code == `ALU_OPC_SUB   ) ? sub_ac  :
(op_code == `ALU_OPC_MUL   ) ? 1'b0    :
(op_code == `ALU_OPC_DIV   ) ? 1'b0    :
(op_code == `ALU_OPC_DA    ) ? 1'b0    :
(op_code == `ALU_OPC_NOT   ) ? 1'b0    :
(op_code == `ALU_OPC_AND   ) ? 1'b0    :
(op_code == `ALU_OPC_XOR   ) ? 1'b0    :
(op_code == `ALU_OPC_OR    ) ? 1'b0    :
(op_code == `ALU_OPC_RL    ) ? 1'b0    :
(op_code == `ALU_OPC_RLC   ) ? 1'b0    :
(op_code == `ALU_OPC_RR    ) ? 1'b0    :
(op_code == `ALU_OPC_RRC   ) ? 1'b0    :
(op_code == `ALU_OPC_PCSADD) ? 1'b0    :
(op_code == `ALU_OPC_PCUADD) ? 1'b0    : 1'b0;
//
assign des_ov_i=
(op_code == `ALU_OPC_ADD   ) ? add_ov  :
(op_code == `ALU_OPC_SUB   ) ? sub_ov  :
//(op_code == `ALU_OPC_MUL   ) ? mul_ov  :
//(op_code == `ALU_OPC_DIV   ) ? div_ov  :
(op_code == `ALU_OPC_MUL   ) ? 1'b0    :
(op_code == `ALU_OPC_DIV   ) ? 1'b0    :
(op_code == `ALU_OPC_DA    ) ? 1'b0    :
(op_code == `ALU_OPC_NOT   ) ? 1'b0    :
(op_code == `ALU_OPC_AND   ) ? 1'b0    :
(op_code == `ALU_OPC_XOR   ) ? 1'b0    :
(op_code == `ALU_OPC_OR    ) ? 1'b0    :
(op_code == `ALU_OPC_RL    ) ? 1'b0    :
(op_code == `ALU_OPC_RLC   ) ? 1'b0    :
(op_code == `ALU_OPC_RR    ) ? 1'b0    :
(op_code == `ALU_OPC_RRC   ) ? 1'b0    :
(op_code == `ALU_OPC_PCSADD) ? 1'b0    :
(op_code == `ALU_OPC_PCUADD) ? 1'b0    : 1'b0;


  always @ (posedge clk or posedge rst) begin
    if(rst) begin
      des_1  <= 8'd0;
      des_2  <= 8'd0;
      des_cy <= 1'b0;
      des_ac <= 1'b0;
      des_ov <= 1'b0;
    end
    else begin
      des_1  <= des_1_i;
      des_2  <= des_2_i;
      des_cy <= des_cy_i;
      des_ac <= des_ac_i;
      des_ov <= des_ov_i;
    end
  end



endmodule

