//
// Copyright (c) 1999-2000 Tony Givargis.  Permission to copy is granted
// provided that this header remains intact.  This software is provided
// with no warranties.
//
// Version : 2.8
//

//////////////////////////////////////////////////////////////////////////////-

//
// rst (active hi) : when asserted the controller state is reset
// clk (rising edge) : clock signal
// rom_addr : this is the rom address being requested
// rom_data : this is the data returned by rom
// rom_rd (active lo) : asserted to signal a rom read
// ram_addr : this is the address of ram/reg being requested
// ram_out_data : this is the data sent to the ram/reg
// ram_in_data : this is the data received from the ram/reg
// ram_out_bit_data : this is the bit-data sent to the ram/reg
// ram_in_bit_data : this is the bit-data received from the ram/reg
// ram_rd (active lo) : asserted to signal a ram/reg read
// ram_wr (active lo) : asserted to signal a ram/reg write
// ram_is_bit_addr (active hi) : asserted if requesting a ram/reg bit-data
// xrm_addr : this is the address of external memory being requested
// xrm_out_data : this is the data sent to the external memory
// xrm_in_data : this is the data received from the external memory
// xrm_rd (active lo) : asserted to signal a external memory read
// xrm_wr (active lo) : asserted to signal a external memory write
// dec_op_out : actual variable length opcode sent to decoder (see 8051 specs)
// dec_op_in(6 downto 0) : cracked (fixed length) opcode (see I8051_LIB)
// dec_op_in(7) : set if this instruction uses a second byte of data
// dec_op_in(8) : set if this instruction uses a third byte of data
// alu_op_code : defines the alu operation requested (see I8051_LIB)
// alu_src_1 : first source operand of the alu (see I8051_ALU)
// alu_src_2 : second source operand of the alu (see I8051_ALU)
// alu_src_3 : third source operand of the alu (see I8051_ALU)
// alu_src_cy : carry into the 7th bit of the alu (see I8051_ALU)
// alu_src_ac : carry into the 4th bit of the alu (see I8051_ALU)
// alu_des_1 : first destination operand of the alu (see I8051_ALU)
// alu_des_2 : second destination operand of the alu (see I8051_ALU)
// alu_des_cy : carry out of the 7th bit of the alu (see I8051_ALU)
// alu_des_ac : carry out of the 4th bit of the alu (see I8051_ALU)
// alu_des_ov : overflow out of the alu (see I8051_ALU)
//
`include "time_scale.v"
`include "i8051_lib.v"
module I8051_CTR (
    rst              ,
    clk              ,
    rom_addr         ,
    rom_data         ,
    rom_rd           ,
    ram_addr         ,
    ram_out_data     ,
    ram_in_data      ,
    ram_out_bit_data ,
    ram_in_bit_data  ,
    ram_rd           ,
    ram_wr           ,
    ram_is_bit_addr  ,
    xrm_addr         ,
    xrm_out_data     ,
    xrm_in_data      ,
    xrm_rd           ,
    xrm_wr           ,
    dec_op_out       ,
    dec_op_in        ,
    alu_op_code      ,
    alu_src_1        ,
    alu_src_2        ,
    alu_src_3        ,
    alu_src_cy       ,
    alu_src_ac       ,
    alu_des_1        ,
    alu_des_2        ,
    alu_des_cy       ,
    alu_des_ac       ,
    alu_des_ov       //
  );
  input          rst;
  input          clk;
  output [11:0]  rom_addr;
  input  [ 7:0]  rom_data;
  output         rom_rd;
  output [ 7:0]  ram_addr;
  output [ 7:0]  ram_out_data;
  input  [ 7:0]  ram_in_data;
  output         ram_out_bit_data;
  input          ram_in_bit_data;
  output         ram_rd;
  output         ram_wr;
  output         ram_is_bit_addr;
  output [15:0]  xrm_addr;
  output [ 7:0]  xrm_out_data;
  input  [ 7:0]  xrm_in_data;
  output         xrm_rd;
  output         xrm_wr;
  output [ 7:0]  dec_op_out;
  input  [ 8:0]  dec_op_in;
  output [ 3:0]  alu_op_code;
  output [ 7:0]  alu_src_1;
  output [ 7:0]  alu_src_2;
  output [ 7:0]  alu_src_3;
  output         alu_src_cy;
  output         alu_src_ac;
  input  [ 7:0]  alu_des_1;
  input  [ 7:0]  alu_des_2;
  input          alu_des_cy;
  input          alu_des_ac;
  input          alu_des_ov;
  
  //////////////////////////////////////////////////////////////////////////////-
  
  
  
  parameter CS_0=2'b00;
  parameter CS_1=2'b01;
  parameter CS_2=2'b10;
  parameter CS_3=2'b11;
  //
  parameter ES_0=3'd0;
  parameter ES_1=3'd1;
  parameter ES_2=3'd2;
  parameter ES_3=3'd3;
  parameter ES_4=3'd4;
  parameter ES_5=3'd5;
  parameter ES_6=3'd6;
  parameter ES_7=3'd7;
 
  reg  [ 3:0]  alu_op_code;
  reg  [ 7:0]  alu_src_1;
  reg  [ 7:0]  alu_src_2;
  reg  [ 7:0]  alu_src_3;
  reg          alu_src_cy;
  reg          alu_src_ac;
  reg  [ 7:0]  ram_addr;
  reg  [ 7:0]  ram_out_data;
  reg          ram_is_bit_addr;
  reg          ram_rd;
  reg          ram_wr;
  reg  [11:0]  rom_addr;
  reg          rom_rd;
  reg  [15:0]  xrm_addr;
  reg  [ 7:0]  xrm_out_data;
  reg          xrm_rd;
  reg          xrm_wr;
  reg          ram_out_bit_data;




  reg  [4:0] reg_pc_15_11 ;
  reg  [2:0] reg_pc_10_8  ;
  reg  [7:0] reg_pc_7_0   ;
  reg  [7:0] reg_op1      ;
  reg  [7:0] reg_op2      ;
  reg  [7:0] reg_op3      ;
  reg  [7:0] reg_acc      ;
  reg        reg_cy       ;
  reg        reg_ac       ;
  reg        reg_f0       ;
  reg        reg_rs1      ;
  reg        reg_rs0      ;
  reg        reg_ov       ;
  reg        reg_nu       ;
  reg        reg_p        ;
  
  reg  [1:0] cpu_state ;
  reg  [2:0] exe_state ;
  
  reg [7:0] v8, pcl, pch;

//wire [7:0] cd_8;
//assign cd_8=`CD_8;

  
  //////////////////////////////////////////////////////////////////////////////-
  
  task SET_PC_1;
    input [7:0] pch, pcl;
    begin
      reg_pc_15_11 = pch[7:3];
      reg_pc_10_8  = pch[2:0];
      reg_pc_7_0   = pcl;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task SET_PC_2;
    input  [2:0]   pch;
    input  [7:0]   pcl;
    begin
      reg_pc_10_8 = pch;
      reg_pc_7_0  = pcl;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task SET_PC_H;
    input  [7:0]   pch;
    begin
      reg_pc_15_11 = pch[7:3];
      reg_pc_10_8  = pch[2:0];
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task SET_PC_L;
    input  [7:0]   pcl;
    begin
      reg_pc_7_0 = pcl;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task GET_PC_H;
    output  [7:0]   pch;
    begin
      pch = {reg_pc_15_11, reg_pc_10_8};
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task GET_PC_L;
    output  [7:0]   pcl;
    begin
      pcl = reg_pc_7_0;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task GET_RAM_ADDR_1;
    output [7:0]   a;
    begin
      a = {3'b000, reg_rs1, reg_rs0, reg_op1[2:0]};
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task GET_RAM_ADDR_2;
    output [7:0]   a;
    begin
      a = {3'b000, reg_rs1, reg_rs0, 2'b00, reg_op1[0]};
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task SET_PSW;
    input [7:0]   p;
    begin
      reg_cy  = p[7];
      reg_ac  = p[6];
      reg_f0  = p[5];
      reg_rs1 = p[4];
      reg_rs0 = p[3];
      reg_ov  = p[2];
      reg_nu  = p[1];
      reg_p   = p[0];
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task GET_PSW;
    output [7:0]   p;
    begin
      p[7] = reg_cy;
      p[6] = reg_ac;
      p[5] = reg_f0;
      p[4] = reg_rs1;
      p[3] = reg_rs0;
      p[2] = reg_ov;
      p[1] = reg_nu;
      p[0] = reg_p;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task START_RD_ROM;
    input [7:0] h, l;
    begin
      rom_addr = {h[3:0], l};
      rom_rd   = 1'b1;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task STOP_RD_ROM;
    begin
      rom_addr = `CD_12;
      rom_rd   = 1'b0;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task START_RD_RAM;
    input  [7:0]   a;
    begin
      ram_addr        = a;
      ram_is_bit_addr = 1'b0;
      ram_rd          = 1'b1;
      ram_wr          = 1'b0;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task START_WR_RAM;
    input  [7:0]   a;
    begin
      ram_addr        = a;
      ram_is_bit_addr = 1'b0;
      ram_rd          = 1'b0;
      ram_wr          = 1'b1;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task START_RD_BIT_RAM;
    input  [7:0]   a;
    begin
      ram_addr        = a;
      ram_is_bit_addr = 1'b1;
      ram_rd          = 1'b1;
      ram_wr          = 1'b0;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task START_WR_BIT_RAM;
    input  [7:0]   a;
    begin
      ram_addr        = a;
      ram_is_bit_addr = 1'b1;
      ram_rd          = 1'b0;
      ram_wr          = 1'b1;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task STOP_RD_WR_RAM;
    begin
      ram_addr         = `CD_8;
      ram_out_data     = `CD_8;
      ram_out_bit_data = 1'b0;
      ram_is_bit_addr  = 1'b0;
      ram_rd           = 1'b0;
      ram_wr           = 1'b0;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task START_RD_XRM;
    input  [15:0]  a;
    begin
      xrm_addr = a;
      xrm_rd   = 1'b1;
      xrm_wr   = 1'b0;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task START_WR_XRM;
    input  [15:0]  a;
    begin
      xrm_addr = a;
      xrm_rd   = 1'b0;
      xrm_wr   = 1'b1;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task STOP_RD_WR_XRM;
    begin
      xrm_addr     = `CD_16;
      xrm_out_data = `CD_8;
      xrm_rd       = 1'b0;
      xrm_wr       = 1'b0;
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task SHUT_DOWN_ALU;
    begin
      alu_op_code = `ALU_OPC_NONE;
      alu_src_1   = `CD_8;
      alu_src_2   = `CD_8;
      alu_src_3   = `CD_8;
      alu_src_cy  = 1'b0;
      alu_src_ac  = 1'b0;
    end//    begin 
  endtask
  
  //
  //////////////////////////////////////////////////////////////////////////////-
  always @ (posedge rst or posedge clk) begin
    if( rst == 1'b1 ) begin
      SET_PC_1(`C0_8, `C0_8);
      reg_op1 = `C0_8;
      reg_op2 = `CD_8;
      reg_op3 = `CD_8;
      reg_acc = `CD_8;
      SET_PSW(`CD_8);
      
      cpu_state = CS_0;
      exe_state = ES_0;
      
      STOP_RD_ROM;
      STOP_RD_WR_RAM;
      STOP_RD_WR_XRM;
      SHUT_DOWN_ALU;

    end//    if( rst == 1'b1 ) begin 
    else begin

      STOP_RD_ROM;
      STOP_RD_WR_RAM;
      STOP_RD_WR_XRM;
      
      case (cpu_state)
        //////////////////////////////////////////////////////////////////////////////-
        
        //
        // reset controller
        //
        CS_0: begin
          case (exe_state)
            ES_0: begin
              ram_out_data = `CM_8;
              START_WR_RAM(`R_P0);
              exe_state = ES_1;
            end//            ES_0: begin 
            ES_1: begin
              ram_out_data = `CM_8;
              START_WR_RAM(`R_P1);
              exe_state = ES_2;
            end//            ES_1: begin 
            ES_2: begin
              ram_out_data = `CM_8;
              START_WR_RAM(`R_P2);
              exe_state = ES_3;
            end//            ES_2: begin 
            ES_3: begin
              ram_out_data = `CM_8;
              START_WR_RAM(`R_P3);
              exe_state = ES_4;
            end//            ES_3: begin 
            ES_4: begin
              ram_out_data = `C7_8;
              START_WR_RAM(`R_SP);
              exe_state = ES_5;
            end//            ES_4: begin 
            ES_5: begin
              cpu_state = CS_1;
              exe_state = ES_0;
            end//            ES_5: begin 
            //default: 
          endcase//          case (exe_state) 
        end//        CS_0: begin 
        
        //////////////////////////////////////////////////////////////////////////////-
        //////////////////////////////////////////////////////////////////////////////-
        
        //
        // handle interrupts
        //
        CS_1: begin
          cpu_state = CS_2;
        end//        CS_1: begin 
        
        //////////////////////////////////////////////////////////////////////////////-
        //////////////////////////////////////////////////////////////////////////////-
        
        //
        // process instructions
        //
        CS_2: begin
          case (exe_state)
            ES_0: begin
              GET_PC_H(pch);
              GET_PC_L(pcl);
              START_RD_ROM(pch, pcl);
              alu_op_code = `ALU_OPC_PCUADD;
              alu_src_1 = pcl;
              alu_src_2 = pch;
              alu_src_3 = `C1_8;
              exe_state = ES_1;
            end//            ES_0: begin 
            ES_1: begin
              START_RD_RAM(`R_PSW);
              exe_state = ES_2;
            end//            ES_1: begin 
            ES_2: begin
              START_RD_RAM(`R_ACC);
              reg_op1 = rom_data;
              exe_state = ES_3;
            end//            ES_2: begin 
            ES_3: begin
              START_RD_ROM(alu_des_2, alu_des_1);
              SET_PSW(ram_in_data);
              alu_op_code = `ALU_OPC_PCUADD;
              alu_src_1 = alu_des_1;
              alu_src_2 = alu_des_2;
              if( dec_op_in[7] == 1'b1 )
              alu_src_3 = `C1_8;
              else
              alu_src_3 = `C0_8;
              exe_state = ES_4;
            end//            ES_3: begin 
            ES_4: begin
              START_RD_ROM(alu_des_2, alu_des_1);
              reg_acc = ram_in_data;
              alu_op_code = `ALU_OPC_PCUADD;
              alu_src_1 = alu_des_1;
              alu_src_2 = alu_des_2;
              if( dec_op_in[8] == 1'b1 )
              alu_src_3 = `C1_8;
              else
              alu_src_3 = `C0_8;
              exe_state = ES_5;
            end//            ES_4: begin 
            ES_5: begin
              reg_op2 = rom_data;
              SET_PC_1(alu_des_2, alu_des_1);
              exe_state = ES_6;
            end//            ES_5: begin 
            ES_6: begin
              reg_op3 = rom_data;
              exe_state = ES_7;
            end//            ES_6: begin 
            ES_7: begin
              SHUT_DOWN_ALU;
              cpu_state = CS_3;
              exe_state = ES_0;
            end//            ES_7: begin 
          endcase//          case (exe_state) 
        end//        CS_2: begin 
        
        //////////////////////////////////////////////////////////////////////////////-
        //////////////////////////////////////////////////////////////////////////////-
        
        //
        // execute state
        //
        CS_3: begin
          case (dec_op_in[6:0])
            
            //////////////////////////////////////////////////////////////////////////////-
            //
            // sp       <- sp + 1
            // mem(sp)  <- pc(7-0)
            // sp       <- sp + 1
            // mem(sp)  <- pc(15-8)
            // pc(10-0) <- page address
            //
            `OPC_ACALL : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_SP);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = alu_des_1;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  ram_out_data = pcl;
                  START_WR_RAM(alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PC_H(pch);
                  ram_out_data = pch;
                  START_WR_RAM(alu_des_1);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_SP);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  SET_PC_2(reg_op1[7:5], reg_op2);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ACALL : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + (r)
            //
            `OPC_ADD_1 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = 1'b0;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ADD_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + (direct)
            //
            `OPC_ADD_2 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = 1'b0;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ADD_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + ((r))
            //
            `OPC_ADD_3 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = 1'b0;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ADD_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + #data
            //
            `OPC_ADD_4 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = reg_op2;
                  alu_src_cy = 1'b0;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ADD_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + cy + (r)
            //
            `OPC_ADDC_1 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = reg_cy;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ADDC_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + cy + (direct)
            //
            `OPC_ADDC_2 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = reg_cy;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ADDC_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + cy + ((r))
            //
            `OPC_ADDC_3 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = reg_cy;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ADDC_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + cy + #data
            //
            `OPC_ADDC_4 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = reg_op2;
                  alu_src_cy = reg_cy;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ADDC_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // pc(10-0) <- page address
            //
            `OPC_AJMP  : begin
              case (exe_state)
                ES_0: begin
                  SET_PC_2(reg_op1[7:5], reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_AJMP  : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc && (r)
            //
            `OPC_ANL_1 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code =`ALU_OPC_AND;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ANL_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc && (direct)
            //
            `OPC_ANL_2 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code =`ALU_OPC_AND;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ANL_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc && ((r))
            //
            `OPC_ANL_3 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  alu_op_code =`ALU_OPC_AND;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ANL_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc && #data
            //
            `OPC_ANL_4 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code =`ALU_OPC_AND;
                  alu_src_1 = reg_acc;
                  alu_src_2 = reg_op2;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ANL_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) && acc
            //
            `OPC_ANL_5 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code =`ALU_OPC_AND;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ANL_5 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) && #data
            //
            `OPC_ANL_6 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code =`ALU_OPC_AND;
                  alu_src_1 = reg_op3;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ANL_6 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // cy <- cy & (bit)
            //
            `OPC_ANL_7 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  reg_cy = reg_cy & ram_in_bit_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ANL_7 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // cy <- cy & ~(bit)
            //
            `OPC_ANL_8 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  reg_cy = reg_cy & (~ram_in_bit_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ANL_8 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( a != (direct) )
            //     pc <- pc + rel
            // if( a < (direct) )
            //     cy <- 1
            // else
            //     cy <- 0
            //
            `OPC_CJNE_1: begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( reg_acc != ram_in_data ) 
                  alu_src_3 = reg_op3;
                  else
                  alu_src_3 = `C0_8;
                  if( reg_acc < ram_in_data )
                  reg_cy = 1'b1;
                  else
                  reg_cy = 1'b0;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            OPC_CJNE_1: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( a != #data )
            //     pc <- pc + rel
            // if( a < #data )
            //     cy <- 1
            // else
            //     cy <- 0
            //
            `OPC_CJNE_2: begin
              case (exe_state)
                ES_0: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( reg_acc != reg_op2 )
                  alu_src_3 = reg_op3;
                  else
                  alu_src_3 = `C0_8;
                  if( reg_acc < reg_op2 )
                  reg_cy = 1'b1;
                  else
                  reg_cy = 1'b0;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            OPC_CJNE_2: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( (r) != #data )
            //     pc <- pc + rel
            // if( (r) < #data )
            //     cy <- 1
            // else
            //     cy <- 0
            //
            `OPC_CJNE_3: begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( ram_in_data != reg_op2 )
                  alu_src_3 = reg_op3;
                  else
                  alu_src_3 = `C0_8;
                  if( ram_in_data < reg_op2 )
                  reg_cy = 1'b1;
                  else
                  reg_cy = 1'b0;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            OPC_CJNE_3: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( ((r)) != #data )
            //     pc <- pc + rel
            // if( ((r)) < #data )
            //     cy <- 1
            // else
            //     cy <- 0
            //
            `OPC_CJNE_4: begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end //                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( ram_in_data != reg_op2 )
                  alu_src_3 = reg_op3;
                  else
                  alu_src_3 = `C0_8;
                  if( ram_in_data < reg_op2 )
                  reg_cy = 1'b1;
                  else
                  reg_cy = 1'b0;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            OPC_CJNE_4: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- 0
            //
            `OPC_CLR_1 : begin
              case (exe_state)
                ES_0: begin
                  ram_out_data = `C0_8;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_CLR_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // cy <- 0
            //
            `OPC_CLR_2 : begin
              case (exe_state)
                ES_0: begin
                  reg_cy = 1'b0;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_CLR_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (bit) <- 0
            //
            `OPC_CLR_3 : begin
              case (exe_state)
                ES_0: begin
                  ram_out_bit_data = 1'b0;
                  START_WR_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_CLR_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- ~acc
            //
            `OPC_CPL_1 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_NOT;
                  alu_src_1 = reg_acc;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_CPL_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // cy <- ~cy
            //
            `OPC_CPL_2 : begin
              case (exe_state)
                ES_0: begin
                  reg_cy = ~reg_cy;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_CPL_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (bit) <- ~(bit)
            //
            `OPC_CPL_3 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_bit_data = ~ram_in_bit_data;
                  START_WR_BIT_RAM(reg_op2);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_CPL_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // see I8051_ALU
            //
            `OPC_DA    : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_DA;
                  alu_src_1 = reg_acc;
                  alu_src_cy = reg_cy;
                  alu_src_ac = reg_ac;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_DA    : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc - 1
            //
            `OPC_DEC_1 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = reg_acc;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_DEC_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (r) <- (r) - 1
            //
            `OPC_DEC_2 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_RAM_ADDR_1(v8);
                  ram_out_data = alu_des_1;
                  START_WR_RAM(v8);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_DEC_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) - 1
            //
            `OPC_DEC_3 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_DEC_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //    
            // ((r)) <- ((r)) - 1
            //    
            `OPC_DEC_4 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(ram_in_data);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_DEC_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // see I8051_ALU
            //
            `OPC_DIV   : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_B);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_DIV;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_ov = alu_des_ov;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_2;
                  START_WR_RAM(`R_B);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_DIV   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (r) <- (r) - 1
            // if( (r) != 0 )
            //     pc <- pc + rel
            //
            `OPC_DJNZ_1: begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  GET_RAM_ADDR_1(v8);
                  ram_out_data = alu_des_1;
                  START_WR_RAM(v8);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( alu_des_1 != `C0_8 )
                  alu_src_3 = reg_op2;
                  else
                  alu_src_3 = `C0_8;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_DJNZ_1: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) - 1
            // if( (direct) != 0 )
            //     pc <- pc + rel
            //
            `OPC_DJNZ_2: begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( alu_des_1 != `C0_8 )
                  alu_src_3 = reg_op3;
                  else
                  alu_src_3 = `C0_8;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_DJNZ_2: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc + 1
            //
            `OPC_INC_1 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = reg_acc;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_INC_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (r) <- (r) + 1
            //
            `OPC_INC_2 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_RAM_ADDR_1(v8);
                  ram_out_data = alu_des_1;
                  START_WR_RAM(v8);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_INC_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) + 1
            //
            `OPC_INC_3 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_INC_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //    
            // ((r)) <- ((r)) + 1
            //    
            //    
            `OPC_INC_4 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(ram_in_data);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_INC_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // dptr <- dptr + 1
            //
            `OPC_INC_5 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_DPL);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  START_RD_RAM(`R_DPH);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_DPL);
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = alu_des_cy;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_DPH);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_INC_5 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( (bit) == 1 )
            //     pc <- pc + rel
            //
            `OPC_JB    : begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( ram_in_bit_data == 1'b1 )
                  alu_src_3 = reg_op3;
                  else
                  alu_src_3 = `C0_8;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_JB    : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( (bit) == 1 )
            //     pc <- pc + rel
            //     (bit) <- 0
            //
            `OPC_JBC   : begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( ram_in_bit_data == 1'b1 ) begin
                    alu_src_3 = reg_op3;
                    ram_out_bit_data = 1'b0;
                    START_WR_BIT_RAM(reg_op2);
                  end//                  if( ram_in_bit_data == 1'b1 ) begin 
                  else begin
                    alu_src_3 = `C0_8;
                  end//                  else begin 
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_JBC   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( cy == 1 )
            //     pc <- pc + rel
            //
            `OPC_JC    : begin
              case (exe_state)
                ES_0: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( reg_cy == 1'b1 )
                  alu_src_3 = reg_op2;
                  else
                  alu_src_3 = `C0_8;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_JC    : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // pc <- dptr + acc
            //
            `OPC_JMP   : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_DPL);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  START_RD_RAM(`R_DPH);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_src_1 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  alu_op_code = `ALU_OPC_PCUADD;
                  alu_src_2 = ram_in_data;
                  alu_src_3 = reg_acc;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_JMP   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( (bit) == 0 )
            //     pc <- pc + rel
            //
            `OPC_JNB   : begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( ram_in_bit_data == 1'b0 )
                  alu_src_3 = reg_op3;
                  else
                  alu_src_3 = `C0_8;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_JNB   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( cy == 0 )
            //     pc <- pc + rel
            //
            `OPC_JNC   : begin
              case (exe_state)
                ES_0: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( reg_cy == 1'b0 )
                  alu_src_3 = reg_op2;
                  else
                  alu_src_3 = `C0_8;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_JNC   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( acc != 0 )
            //     pc <- pc + rel
            //
            `OPC_JNZ   : begin
              case (exe_state)
                ES_0: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( reg_acc != `C0_8 )
                  alu_src_3 = reg_op2;
                  else
                  alu_src_3 = `C0_8;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_JNZ   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // if( acc == 0 )
            //     pc <- pc + rel
            //
            `OPC_JZ    : begin
              case (exe_state)
                ES_0: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  if( reg_acc == `C0_8 )
                  alu_src_3 = reg_op2;
                  else
                  alu_src_3 = `C0_8;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_JZ    : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // sp       <- sp + 1
            // mem(sp)  <- pc(7-0)
            // sp       <- sp + 1
            // mem(sp)  <- pc(15-8)
            // pc(15-0) <- address
            //
            `OPC_LCALL : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_SP);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = alu_des_1;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  ram_out_data = pcl;
                  START_WR_RAM(alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PC_H(pch);
                  ram_out_data = pch;
                  START_WR_RAM(alu_des_1);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_SP);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  SET_PC_1(reg_op2, reg_op3);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_LCALL : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // pc(15-0) <- address
            //
            `OPC_LJMP  : begin
              case (exe_state)
                ES_0: begin
                  SET_PC_1(reg_op2, reg_op3);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_LJMP  : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- (r)
            //
            `OPC_MOV_1 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- (direct)
            //
            `OPC_MOV_2 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- ((r))
            //
            `OPC_MOV_3 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- #data
            //
            `OPC_MOV_4 : begin
              case (exe_state)
                ES_0: begin
                  ram_out_data = reg_op2;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (r) <- acc
            //
            `OPC_MOV_5 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  ram_out_data = reg_acc;
                  START_WR_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_5 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (r) <- (direct)
            //
            `OPC_MOV_6 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_RAM_ADDR_1(v8);
                  ram_out_data = ram_in_data;
                  START_WR_RAM(v8);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_6 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (r) <- #data
            //
            `OPC_MOV_7 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  ram_out_data = reg_op2;
                  START_WR_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_7 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- acc
            //
            `OPC_MOV_8 : begin
              case (exe_state)
                ES_0: begin
                  ram_out_data = reg_acc;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_8 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (r)
            //
            `OPC_MOV_9 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_9 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct)
            //
            `OPC_MOV_10: begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(reg_op3);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_10: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- ((r))
            //
            `OPC_MOV_11: begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_11: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- #data
            //
            `OPC_MOV_12: begin
              case (exe_state)
                ES_0: begin
                  ram_out_data = reg_op3;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_12: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // ((r)) <- acc
            //
            `OPC_MOV_13: begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_data = reg_acc;
                  START_WR_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_13: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // ((r)) <- (direct)
            //
            `OPC_MOV_14: begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  reg_acc = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = reg_acc;
                  START_WR_RAM(ram_in_data);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_14: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // ((r)) <- #data
            //
            `OPC_MOV_15: begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_data = reg_op2;
                  START_WR_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_15: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // cy <- (bit)
            //
            `OPC_MOV_16: begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  reg_cy = ram_in_bit_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_16: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (bit) <- cy
            //
            `OPC_MOV_17: begin
              case (exe_state)
                ES_0: begin
                  ram_out_bit_data = reg_cy;
                  START_WR_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_17: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // dph <- #data-hi
            // dpl <- #data-lo
            //
            `OPC_MOV_18: begin
              case (exe_state)
                ES_0: begin
                  ram_out_data = reg_op2;
                  START_WR_RAM(`R_DPH);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = reg_op3;
                  START_WR_RAM(`R_DPL);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOV_18: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- (dptr + acc)
            //
            `OPC_MOVC_1: begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_DPL);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  START_RD_RAM(`R_DPH);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_src_1 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  alu_op_code = `ALU_OPC_PCUADD;
                  alu_src_2 = ram_in_data;
                  alu_src_3 = reg_acc;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  START_RD_ROM(alu_des_2, alu_des_1);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  ram_out_data = rom_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOVC_1: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- (pc + acc)
            //
            `OPC_MOVC_2: begin
              case (exe_state)
                ES_0: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCUADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  alu_src_3 = reg_acc;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  START_RD_ROM(alu_des_2, alu_des_1);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = rom_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOVC_2: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- ((r))
            //
            `OPC_MOVX_1: begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_XRM({8'b0000_0000, ram_in_data});
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = xrm_in_data;
                  START_WR_RAM(reg_acc);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOVX_1: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- (DPTR)
            //
            `OPC_MOVX_2: begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_DPL);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  START_RD_RAM(`R_DPH);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_src_1 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  alu_op_code = `ALU_OPC_PCUADD;
                  alu_src_2 = ram_in_data;
                  alu_src_3 = `C0_8;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  START_RD_XRM({alu_des_2, alu_des_1});
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  ram_out_data = xrm_in_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOVX_2: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // ((r)) <- acc
            //
            `OPC_MOVX_3: begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  xrm_out_data = reg_acc;
                  START_WR_XRM({8'b0000_0000, ram_in_data});
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOVX_3: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (DPTR) <- acc
            //
            `OPC_MOVX_4: begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_DPL);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  START_RD_RAM(`R_DPH);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_src_1 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  alu_op_code = `ALU_OPC_PCUADD;
                  alu_src_2 = ram_in_data;
                  alu_src_3 = `C0_8;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  xrm_out_data = reg_acc;
                  START_WR_XRM({alu_des_2, alu_des_1});
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MOVX_4: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // see I8051_ALU
            //
            `OPC_MUL   : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_B);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_MUL;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_ov = alu_des_ov;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_2;
                  START_WR_RAM(`R_B);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_MUL   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // no operation
            //
            `OPC_NOP   : begin
              case (exe_state)
                ES_0: begin
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_NOP   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc || (r)
            //
            `OPC_ORL_1 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_OR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ORL_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc || (direct)
            //
            `OPC_ORL_2 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_OR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ORL_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc || ((r))
            //
            `OPC_ORL_3 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  alu_op_code = `ALU_OPC_OR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ORL_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc || #data
            //
            `OPC_ORL_4 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_OR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = reg_op2;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ORL_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) || acc
            //
            `OPC_ORL_5 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_OR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ORL_5 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) || #data
            //
            `OPC_ORL_6 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_OR;
                  alu_src_1 = reg_op3;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ORL_6 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // cy <- cy | (bit)
            //
            `OPC_ORL_7 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  reg_cy = reg_cy | ram_in_bit_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ORL_7 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // cy <- cy | ~(bit)
            //
            `OPC_ORL_8 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  reg_cy = reg_cy | (~ram_in_bit_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_ORL_8 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (sp)
            // sp <- sp - 1
            //
            `OPC_POP   : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_SP);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_SP);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_POP   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // sp <- sp + 1
            // (sp) <- (direct)
            //
            `OPC_PUSH  : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_SP);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_SP);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_PUSH  : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // pc(15-8) <- (sp)
            // sp <- sp - 1
            // pc(7-0) <- (sp)
            // sp <- sp - 1
            //
            `OPC_RET   : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(`R_SP);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  START_RD_RAM(alu_des_1);
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = alu_des_1;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b1;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  SET_PC_H(ram_in_data);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  SET_PC_L(ram_in_data);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_SP);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_RET   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // see I8051_ALU
            //
            `OPC_RL    : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_RL;
                  alu_src_1 = reg_acc;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_RL    : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // see I8051_ALU
            //
            `OPC_RLC   : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_RLC;
                  alu_src_1 = reg_acc;
                  alu_src_cy = reg_cy;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_RLC   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // see I8051_ALU
            //
            `OPC_RR    : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_RR;
                  alu_src_1 = reg_acc;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_RR    : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // see I8051_ALU
            //
            `OPC_RRC   : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_RRC;
                  alu_src_1 = reg_acc;
                  alu_src_cy = reg_cy;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_RRC   : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // cy <- 1
            //
            `OPC_SETB_1: begin
              case (exe_state)
                ES_0: begin
                  reg_cy = 1'b1;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_SETB_1: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (bit) <- 1
            //
            `OPC_SETB_2: begin
              case (exe_state)
                ES_0: begin
                  ram_out_bit_data = 1'b1;
                  START_WR_BIT_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_SETB_2: begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // pc <- pc + rel
            //
            `OPC_SJMP  : begin
              case (exe_state)
                ES_0: begin
                  GET_PC_H(pch);
                  GET_PC_L(pcl);
                  alu_op_code = `ALU_OPC_PCSADD;
                  alu_src_1 = pcl;
                  alu_src_2 = pch;
                  alu_src_3 = reg_op2;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  SET_PC_1(alu_des_2, alu_des_1);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_SJMP  : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc - cy - (r)
            //
            `OPC_SUBB_1 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = reg_cy;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_SUBB_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc - cy - (direct)
            //
            `OPC_SUBB_2 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = reg_cy;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_SUBB_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc - cy - ((r))
            //
            `OPC_SUBB_3 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  alu_src_cy = reg_cy;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_SUBB_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc - cy - #data
            //
            `OPC_SUBB_4 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_SUB;
                  alu_src_1 = reg_acc;
                  alu_src_2 = reg_op2;
                  alu_src_cy = reg_cy;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  reg_cy = alu_des_cy;
                  reg_ac = alu_des_ac;
                  reg_ov = alu_des_ov;
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  GET_PSW(v8);
                  ram_out_data = v8;
                  START_WR_RAM(`R_PSW);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_SUBB_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc(3-0) <-> acc(7-4)
            //
            `OPC_SWAP  : begin
              case (exe_state)
                ES_0: begin
                  ram_out_data = {reg_acc[3:0], reg_acc[7:4]};
                  
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_SWAP  : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <-> (r)
            //
            `OPC_XCH_1 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  GET_RAM_ADDR_1(v8);
                  ram_out_data = reg_acc;
                  START_WR_RAM(v8);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XCH_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <-> (direct)
            //
            `OPC_XCH_2 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = reg_acc;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XCH_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <-> ((r))
            //
            `OPC_XCH_3 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = reg_acc;
                  START_WR_RAM(ram_in_data);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = ram_in_data;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XCH_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc(3-0) <-> ((r))(3-0)
            //
            `OPC_XCHD  : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_ADD;
                  alu_src_1 = ram_in_data;
                  alu_src_2 = `C0_8;
                  alu_src_cy = 1'b0;
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  START_RD_RAM(alu_des_1);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  ram_out_data = {ram_in_data[7:4], reg_acc[3:0]};
                  START_WR_RAM(alu_des_1);
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data ={reg_acc[7:4], ram_in_data[3:0]};
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XCHD  : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc ^ (r)
            //
            `OPC_XRL_1 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_1(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_XOR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XRL_1 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc ^ (direct)
            //
            `OPC_XRL_2 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_XOR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XRL_2 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc ^ ((r))
            //
            `OPC_XRL_3 : begin
              case (exe_state)
                ES_0: begin
                  GET_RAM_ADDR_2(v8);
                  START_RD_RAM(v8);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  START_RD_RAM(ram_in_data);
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  alu_op_code = `ALU_OPC_XOR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XRL_3 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // acc <- acc ^ #data
            //
            `OPC_XRL_4 : begin
              case (exe_state)
                ES_0: begin
                  alu_op_code = `ALU_OPC_XOR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = reg_op2;
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(`R_ACC);
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XRL_4 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) ^ acc
            //
            `OPC_XRL_5 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_XOR;
                  alu_src_1 = reg_acc;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XRL_5 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
            //
            // (direct) <- (direct) ^ #data
            //
            `OPC_XRL_6 : begin
              case (exe_state)
                ES_0: begin
                  START_RD_RAM(reg_op2);
                  exe_state = ES_1;
                end//                ES_0: begin 
                ES_1: begin
                  exe_state = ES_2;
                end//                ES_1: begin 
                ES_2: begin
                  alu_op_code = `ALU_OPC_XOR;
                  alu_src_1 = reg_op3;
                  alu_src_2 = ram_in_data;
                  exe_state = ES_3;
                end//                ES_2: begin 
                ES_3: begin
                  ram_out_data = alu_des_1;
                  START_WR_RAM(reg_op2);
                  exe_state = ES_4;
                end//                ES_3: begin 
                ES_4: begin
                  exe_state = ES_5;
                end//                ES_4: begin 
                ES_5: begin
                  exe_state = ES_6;
                end//                ES_5: begin 
                ES_6: begin
                  exe_state = ES_7;
                end//                ES_6: begin 
                ES_7: begin
                  SHUT_DOWN_ALU;
                  cpu_state = CS_1;
                  exe_state = ES_0;
                end//                ES_7: begin 
              endcase//              case (exe_state) 
            end//            `OPC_XRL_6 : begin 
            
            //////////////////////////////////////////////////////////////////////////////-
            
          endcase//          case (dec_op_in[6:0]) 
        end//        CS_3: begin 
      endcase//      case (cpu_state) 
    end//    else begin 
  end//  always @ (posedge rst or posedge clk) begin 
  assign dec_op_out = reg_op1;
endmodule// 

//////////////////////////////////////////////////////////////////////////////-


