//
// Copyright (c) 1999-2000 Tony Givargis.  Permission to copy is granted
// provided that this header remains intact.  This software is provided
// with no warranties.
//
// Version : 2.8
//

//////////////////////////////////////////////////////////////////////////////-


//
// rst (active hi) : OPC_ERROR returned when this is asserted
// op_in : actual variable length opcode (see 8051 specs)
// op_out(6 downto 0) : cracked (fixed length) opcode (see I8051_LIB)
// op_out(7) : set if this instruction uses a second byte of data
// op_out(8) : set if this instruction uses a third byte of data
//
`include "time_scale.v"
`include "i8051_lib.v"
module I8051_DEC (rst, op_in, op_out);
input        rst     ;
input  [7:0] op_in   ;
output [8:0] op_out  ;

//////////////////////////////////////////////////////////////////////////////-

assign op_out = 
(rst        == 1'b1    )? {2'b00, `OPC_ERROR } :
(op_in[4:0] == `ACALL  ) ? {2'b01, `OPC_ACALL } :
(op_in[7:3] == `ADD_1  ) ? {2'b00, `OPC_ADD_1 } :
(op_in[7:0] == `ADD_2  ) ? {2'b01, `OPC_ADD_2 } :
(op_in[7:1] == `ADD_3  ) ? {2'b00, `OPC_ADD_3 } :
(op_in[7:0] == `ADD_4  ) ? {2'b01, `OPC_ADD_4 } :
(op_in[7:3] == `ADDC_1 ) ? {2'b00, `OPC_ADDC_1} :
(op_in[7:0] == `ADDC_2 ) ? {2'b01, `OPC_ADDC_2} :
(op_in[7:1] == `ADDC_3 ) ? {2'b00, `OPC_ADDC_3} :
(op_in[7:0] == `ADDC_4 ) ? {2'b01, `OPC_ADDC_4} :
(op_in[4:0] == `AJMP   ) ? {2'b01, `OPC_AJMP  } :
(op_in[7:3] == `ANL_1  ) ? {2'b00, `OPC_ANL_1 } :
(op_in[7:0] == `ANL_2  ) ? {2'b01, `OPC_ANL_2 } :
(op_in[7:1] == `ANL_3  ) ? {2'b00, `OPC_ANL_3 } :
(op_in[7:0] == `ANL_4  ) ? {2'b01, `OPC_ANL_4 } :
(op_in[7:0] == `ANL_5  ) ? {2'b01, `OPC_ANL_5 } :
(op_in[7:0] == `ANL_6  ) ? {2'b11, `OPC_ANL_6 } :
(op_in[7:0] == `ANL_7  ) ? {2'b01, `OPC_ANL_7 } :
(op_in[7:0] == `ANL_8  ) ? {2'b01, `OPC_ANL_8 } :
(op_in[7:0] == `CJNE_1 ) ? {2'b11, `OPC_CJNE_1} :
(op_in[7:0] == `CJNE_2 ) ? {2'b11, `OPC_CJNE_2} :
(op_in[7:3] == `CJNE_3 ) ? {2'b11, `OPC_CJNE_3} :
(op_in[7:1] == `CJNE_4 ) ? {2'b11, `OPC_CJNE_4} :
(op_in[7:0] == `CLR_1  ) ? {2'b00, `OPC_CLR_1 } :
(op_in[7:0] == `CLR_2  ) ? {2'b00, `OPC_CLR_2 } :
(op_in[7:0] == `CLR_3  ) ? {2'b01, `OPC_CLR_3 } :
(op_in[7:0] == `CPL_1  ) ? {2'b00, `OPC_CPL_1 } :
(op_in[7:0] == `CPL_2  ) ? {2'b00, `OPC_CPL_2 } :
(op_in[7:0] == `CPL_3  ) ? {2'b01, `OPC_CPL_3 } :
(op_in[7:0] == `DA     ) ? {2'b00, `OPC_DA    } :
(op_in[7:0] == `DEC_1  ) ? {2'b00, `OPC_DEC_1 } :
(op_in[7:3] == `DEC_2  ) ? {2'b00, `OPC_DEC_2 } :
(op_in[7:0] == `DEC_3  ) ? {2'b01, `OPC_DEC_3 } :
(op_in[7:1] == `DEC_4  ) ? {2'b00, `OPC_DEC_4 } :
(op_in[7:0] == `DIV    ) ? {2'b00, `OPC_DIV   } :
(op_in[7:3] == `DJNZ_1 ) ? {2'b01, `OPC_DJNZ_1} :
(op_in[7:0] == `DJNZ_2 ) ? {2'b11, `OPC_DJNZ_2} :
(op_in[7:0] == `INC_1  ) ? {2'b00, `OPC_INC_1 } :
(op_in[7:3] == `INC_2  ) ? {2'b00, `OPC_INC_2 } :
(op_in[7:0] == `INC_3  ) ? {2'b01, `OPC_INC_3 } :
(op_in[7:1] == `INC_4  ) ? {2'b00, `OPC_INC_4 } :
(op_in[7:0] == `INC_5  ) ? {2'b00, `OPC_INC_5 } :
(op_in[7:0] == `JB     ) ? {2'b11, `OPC_JB    } :
(op_in[7:0] == `JBC    ) ? {2'b11, `OPC_JBC   } :
(op_in[7:0] == `JC     ) ? {2'b01, `OPC_JC    } :
(op_in[7:0] == `JMP    ) ? {2'b00, `OPC_JMP   } :
(op_in[7:0] == `JNB    ) ? {2'b11, `OPC_JNB   } :
(op_in[7:0] == `JNC    ) ? {2'b01, `OPC_JNC   } :
(op_in[7:0] == `JNZ    ) ? {2'b01, `OPC_JNZ   } :
(op_in[7:0] == `JZ     ) ? {2'b01, `OPC_JZ    } :
(op_in[7:0] == `LCALL  ) ? {2'b11, `OPC_LCALL } :
(op_in[7:0] == `LJMP   ) ? {2'b11, `OPC_LJMP  } :
(op_in[7:3] == `MOV_1  ) ? {2'b00, `OPC_MOV_1 } :
(op_in[7:0] == `MOV_2  ) ? {2'b01, `OPC_MOV_2 } :
(op_in[7:1] == `MOV_3  ) ? {2'b00, `OPC_MOV_3 } :
(op_in[7:0] == `MOV_4  ) ? {2'b01, `OPC_MOV_4 } :
(op_in[7:3] == `MOV_5  ) ? {2'b00, `OPC_MOV_5 } :
(op_in[7:3] == `MOV_6  ) ? {2'b01, `OPC_MOV_6 } :
(op_in[7:3] == `MOV_7  ) ? {2'b01, `OPC_MOV_7 } :
(op_in[7:0] == `MOV_8  ) ? {2'b01, `OPC_MOV_8 } :
(op_in[7:3] == `MOV_9  ) ? {2'b01, `OPC_MOV_9 } :
(op_in[7:0] == `MOV_10 ) ? {2'b11, `OPC_MOV_10} :
(op_in[7:1] == `MOV_11 ) ? {2'b01, `OPC_MOV_11} :
(op_in[7:0] == `MOV_12 ) ? {2'b11, `OPC_MOV_12} :
(op_in[7:1] == `MOV_13 ) ? {2'b00, `OPC_MOV_13} :
(op_in[7:1] == `MOV_14 ) ? {2'b01, `OPC_MOV_14} :
(op_in[7:1] == `MOV_15 ) ? {2'b01, `OPC_MOV_15} :
(op_in[7:0] == `MOV_16 ) ? {2'b01, `OPC_MOV_16} :
(op_in[7:0] == `MOV_17 ) ? {2'b01, `OPC_MOV_17} :
(op_in[7:0] == `MOV_18 ) ? {2'b11, `OPC_MOV_18} :
(op_in[7:0] == `MOVC_1 ) ? {2'b00, `OPC_MOVC_1} :
(op_in[7:0] == `MOVC_2 ) ? {2'b00, `OPC_MOVC_2} :
(op_in[7:1] == `MOVX_1 ) ? {2'b00, `OPC_MOVX_1} :
(op_in[7:0] == `MOVX_2 ) ? {2'b00, `OPC_MOVX_2} :
(op_in[7:1] == `MOVX_3 ) ? {2'b00, `OPC_MOVX_3} :
(op_in[7:0] == `MOVX_4 ) ? {2'b00, `OPC_MOVX_4} :
(op_in[7:0] == `MUL    ) ? {2'b00, `OPC_MUL   } :
(op_in[7:0] == `NOP    ) ? {2'b00, `OPC_NOP   } :
(op_in[7:3] == `ORL_1  ) ? {2'b00, `OPC_ORL_1 } :
(op_in[7:0] == `ORL_2  ) ? {2'b01, `OPC_ORL_2 } :
(op_in[7:1] == `ORL_3  ) ? {2'b00, `OPC_ORL_3 } :
(op_in[7:0] == `ORL_4  ) ? {2'b01, `OPC_ORL_4 } :
(op_in[7:0] == `ORL_5  ) ? {2'b01, `OPC_ORL_5 } :
(op_in[7:0] == `ORL_6  ) ? {2'b11, `OPC_ORL_6 } :
(op_in[7:0] == `ORL_7  ) ? {2'b01, `OPC_ORL_7 } :
(op_in[7:0] == `ORL_8  ) ? {2'b01, `OPC_ORL_8 } :
(op_in[7:0] == `POP    ) ? {2'b01, `OPC_POP   } :
(op_in[7:0] == `PUSH   ) ? {2'b01, `OPC_PUSH  } :
(op_in[7:0] == `RET    ) ? {2'b00, `OPC_RET   } :
(op_in[7:0] == `RETI   ) ? {2'b00, `OPC_RETI  } :
(op_in[7:0] == `RL     ) ? {2'b00, `OPC_RL    } :
(op_in[7:0] == `RLC    ) ? {2'b00, `OPC_RLC   } :
(op_in[7:0] == `RR     ) ? {2'b00, `OPC_RR    } :
(op_in[7:0] == `RRC    ) ? {2'b00, `OPC_RRC   } :
(op_in[7:0] == `SETB_1 ) ? {2'b00, `OPC_SETB_1} :
(op_in[7:0] == `SETB_2 ) ? {2'b01, `OPC_SETB_2} :
(op_in[7:0] == `SJMP   ) ? {2'b01, `OPC_SJMP  } :
(op_in[7:3] == `SUBB_1 ) ? {2'b00, `OPC_SUBB_1} :
(op_in[7:0] == `SUBB_2 ) ? {2'b01, `OPC_SUBB_2} :
(op_in[7:1] == `SUBB_3 ) ? {2'b00, `OPC_SUBB_3} :
(op_in[7:0] == `SUBB_4 ) ? {2'b01, `OPC_SUBB_4} :
(op_in[7:0] == `SWAP   ) ? {2'b00, `OPC_SWAP  } :
(op_in[7:3] == `XCH_1  ) ? {2'b00, `OPC_XCH_1 } :
(op_in[7:0] == `XCH_2  ) ? {2'b01, `OPC_XCH_2 } :
(op_in[7:1] == `XCH_3  ) ? {2'b00, `OPC_XCH_3 } :
(op_in[7:1] == `XCHD   ) ? {2'b00, `OPC_XCHD  } :
(op_in[7:3] == `XRL_1  ) ? {2'b00, `OPC_XRL_1 } :
(op_in[7:0] == `XRL_2  ) ? {2'b01, `OPC_XRL_2 } :
(op_in[7:1] == `XRL_3  ) ? {2'b00, `OPC_XRL_3 } :
(op_in[7:0] == `XRL_4  ) ? {2'b01, `OPC_XRL_4 } :
(op_in[7:0] == `XRL_5  ) ? {2'b01, `OPC_XRL_5 } :
(op_in[7:0] == `XRL_6  ) ? {2'b11, `OPC_XRL_6 } :
{2'b00, `OPC_ERROR };

endmodule
//////////////////////////////////////////////////////////////////////////////-

// end of file //

