//
// Copyright (c) 1999-2000 Tony Givargis.  Permission to copy is granted
// provided that this header remains intact.  This software is provided
// with no warranties.
//
// Version : 2.8
//

//////////////////////////////////////////////////////////////////////////////-


//
// rst (active hi) : when asserted, the registers are set to default values
// clk (rising edge) : clock signal - all ram i/o is synchronous
// addr : this is the address of ram/reg being requested
// in_data : this is the data being writen into the ram/reg
// out_data : this is the data being read from the ram/reg
// in_bit_data : this is the bit-data being writen into the ram/reg
// out_bit_data : this is the bit-data being read from the ram/reg
// rd (active lo) : asserted to signal a ram/reg read
// wr (active lo) : asserted to signal a ram/reg write
// is_bit_addr (active hi) : asserted if requesting a ram/reg bit-data
// p0_in : write access to the 8051's port 0
// p0_out : read access to the 8051's port 0
// p1_in : write access to the 8051's port 0
// p1_out : read access to the 8051's port 0
// p2_in : write access to the 8051's port 0
// p2_out : read access to the 8051's port 0
// p3_in : write access to the 8051's port 0
// p3_out : read access to the 8051's port 0
//
`include "time_scale.v"
`include "i8051_lib.v"
module I8051_RAM (
    rst          ,
    clk          ,
    addr         ,
    in_data      ,
    out_data     ,
    in_bit_data  ,
    out_bit_data ,
    rd           ,
    wr           ,
    is_bit_addr  ,
    p0_in        ,
    p0_out       ,
    p1_in        ,
    p1_out       ,
    p2_in        ,
    p2_out       ,
    p3_in        ,
    p3_out       //
  );
  input          rst;
  input          clk;
  input  [7:0]   addr;
  input  [7:0]   in_data;
  output [7:0]   out_data;
  input          in_bit_data;
  output         out_bit_data;
  input          rd;
  input          wr;
  input          is_bit_addr;
  input  [7:0]   p0_in;
  output [7:0]   p0_out;
  input  [7:0]   p1_in;
  output [7:0]   p1_out;
  input  [7:0]   p2_in;
  output [7:0]   p2_out;
  input  [7:0]   p3_in;
  output [7:0]   p3_out;

  reg  [7:0]   p0_out;
  reg  [7:0]   p1_out;
  reg  [7:0]   p2_out;
  reg  [7:0]   p3_out;
  reg  [7:0]   out_data;
  reg          out_bit_data;
  
  
  //////////////////////////////////////////////////////////////////////////////-
  
  reg  [7:0] sfr_b    ;
  reg  [7:0] sfr_acc  ;
  reg  [7:0] sfr_psw  ;
  reg  [7:0] sfr_ie   ;
  reg  [7:0] sfr_ip   ;
  reg  [7:0] sfr_sp   ;
  reg  [7:0] sfr_dpl  ;
  reg  [7:0] sfr_dph  ;
  reg  [7:0] sfr_pcon ;
  reg  [7:0] sfr_scon ;
  reg  [7:0] sfr_sbuf ;
  reg  [7:0] sfr_tcon ;
  reg  [7:0] sfr_tmod ;
  reg  [7:0] sfr_tl0  ;
  reg  [7:0] sfr_th0  ;
  reg  [7:0] sfr_tl1  ;
  reg  [7:0] sfr_th1  ;
  
  //Pini
  wire [7:0] r_o  ;
  wire [6:0] r_a  ;
  reg  [6:0] r_ar ;
  wire [7:0] r_d  ;
  reg  [7:0] r_dr ;
  wire       r_we ;
  reg        r_wer;
  
  
  assign r_we=r_wer;
  assign r_a =r_ar;
  assign r_d =r_dr;

  RAM128X1S_1 u_ram0 (
    .O (r_o[0]),
    .A0(r_a[0]),
    .A1(r_a[1]),
    .A2(r_a[2]),
    .A3(r_a[3]),
    .A4(r_a[4]),
    .A5(r_a[5]),
    .A6(r_a[6]),
    .D (r_d[0]),
    .WCLK(clk ),
    .WE  (r_we)
  );//u_ram0 : RAM128X1S_1
  RAM128X1S_1 u_ram1 (
    .O (r_o[1]),
    .A0(r_a[0]),
    .A1(r_a[1]),
    .A2(r_a[2]),
    .A3(r_a[3]),
    .A4(r_a[4]),
    .A5(r_a[5]),
    .A6(r_a[6]),
    .D (r_d[1]),
    .WCLK(clk ),
    .WE  (r_we)
  );//u_ram1 : RAM128X1S_1
  RAM128X1S_1 u_ram2 (
    .O (r_o[2]),
    .A0(r_a[0]),
    .A1(r_a[1]),
    .A2(r_a[2]),
    .A3(r_a[3]),
    .A4(r_a[4]),
    .A5(r_a[5]),
    .A6(r_a[6]),
    .D (r_d[2]),
    .WCLK(clk ),
    .WE  (r_we)
  );//u_ram2 : RAM128X1S_1
  RAM128X1S_1 u_ram3 (
    .O (r_o[3]),
    .A0(r_a[0]),
    .A1(r_a[1]),
    .A2(r_a[2]),
    .A3(r_a[3]),
    .A4(r_a[4]),
    .A5(r_a[5]),
    .A6(r_a[6]),
    .D (r_d[3]),
    .WCLK(clk ),
    .WE  (r_we)
  );//u_ram3 : RAM128X1S_1
  RAM128X1S_1 u_ram4 (
    .O (r_o[4]),
    .A0(r_a[0]),
    .A1(r_a[1]),
    .A2(r_a[2]),
    .A3(r_a[3]),
    .A4(r_a[4]),
    .A5(r_a[5]),
    .A6(r_a[6]),
    .D (r_d[4]),
    .WCLK(clk ),
    .WE  (r_we)
  );//u_ram4 : RAM128X1S_1
  RAM128X1S_1 u_ram5 (
    .O (r_o[5]),
    .A0(r_a[0]),
    .A1(r_a[1]),
    .A2(r_a[2]),
    .A3(r_a[3]),
    .A4(r_a[4]),
    .A5(r_a[5]),
    .A6(r_a[6]),
    .D (r_d[5]),
    .WCLK(clk ),
    .WE  (r_we)
  );//u_ram5 : RAM128X1S_1
  RAM128X1S_1 u_ram6 (
    .O (r_o[6]),
    .A0(r_a[0]),
    .A1(r_a[1]),
    .A2(r_a[2]),
    .A3(r_a[3]),
    .A4(r_a[4]),
    .A5(r_a[5]),
    .A6(r_a[6]),
    .D (r_d[6]),
    .WCLK(clk ),
    .WE  (r_we)
  );//u_ram6 : RAM128X1S_1
  RAM128X1S_1 u_ram7 (
    .O (r_o[7]),
    .A0(r_a[0]),
    .A1(r_a[1]),
    .A2(r_a[2]),
    .A3(r_a[3]),
    .A4(r_a[4]),
    .A5(r_a[5]),
    .A6(r_a[6]),
    .D (r_d[7]),
    .WCLK(clk ),
    .WE  (r_we)
  );//u_ram7 : RAM128X1S_1
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task GET_BYTE;
    input  [7:0]   a;
    output [7:0]   v;
    reg    [7:0]   v;
    begin
      r_wer = 1'b0;//Pini
      case (a)
        `R_B     : v = sfr_b;
        `R_ACC   : v = sfr_acc;
        `R_PSW   : v = sfr_psw;
        `R_IP    : v = sfr_ip;
        `R_IE    : v = sfr_ie;
        `R_SP    : v = sfr_sp;
        `R_P0    : v = p0_in;
        `R_P1    : v = p1_in;
        `R_P2    : v = p2_in;
        `R_P3    : v = p3_in;
        `R_DPL   : v = sfr_dpl;
        `R_DPH   : v = sfr_dph;
        `R_PCON  : v = sfr_pcon;
        `R_SCON  : v = sfr_scon;
        `R_SBUF  : v = sfr_sbuf;
        `R_TCON  : v = sfr_tcon;
        `R_TMOD  : v = sfr_tmod;
        `R_TL0   : v = sfr_tl0;
        `R_TL1   : v = sfr_tl1;
        `R_TH0   : v = sfr_th0;
        `R_TH1   : v = sfr_th1;
        //Pini
        //`others  : v = iram(conv_integer(a[6:0]));
        default  : begin 
          v = r_o; r_ar = a[6:0];
        end//        default  : begin  
      endcase//      case (a) 
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task SET_BYTE;
    input  [7:0]   a;
    input  [7:0]   v;
    begin
      r_wer = 1'b0;//Pini
      case (a)
        `R_B     : sfr_b    = v;
        `R_ACC   : sfr_acc  = v;
        `R_PSW   : sfr_psw  = v;
        `R_IP    : sfr_ip   = v;
        `R_IE    : sfr_ie   = v;
        `R_SP    : sfr_sp   = v;
        `R_P0    : p0_out   = v;
        `R_P1    : p1_out   = v;
        `R_P2    : p2_out   = v;
        `R_P3    : p3_out   = v;
        `R_DPL   : sfr_dpl  = v;
        `R_DPH   : sfr_dph  = v;
        `R_PCON  : sfr_pcon = v;
        `R_SCON  : sfr_scon = v;
        `R_SBUF  : sfr_sbuf = v;
        `R_TCON  : sfr_tcon = v;
        `R_TMOD  : sfr_tmod = v;
        `R_TL0   : sfr_tl0  = v;
        `R_TL1   : sfr_tl1  = v;
        `R_TH0   : sfr_th0  = v;
        `R_TH1   : sfr_th1  = v;
        //Pini
        //`others  : iram(conv_integer(a[6:0])) = v;
        default  : begin 
          r_dr = v; r_wer = 1'b1; r_ar = a[6:0];
        end//        default  : begin  
      endcase//      case (a) 
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task GET_BIT;
    input  [7:0] a; 
    output       v;
    reg          v;
    reg    [7:0] vu;
    integer      vi;
    begin
      r_wer = 1'b0;//Pini
      vu   = {a[7:3], 3'b000};
      vi   = a[2:0];
      case (vu)
        `R_B     : v = sfr_b[vi];
        `R_ACC   : v = sfr_acc[vi];
        `R_PSW   : v = sfr_psw[vi];
        `R_IP    : v = sfr_ip[vi];
        `R_IE    : v = sfr_ie[vi];
        `R_SP    : v = sfr_sp[vi];
        `R_P0    : v = p0_in[vi];
        `R_P1    : v = p1_in[vi];
        `R_P2    : v = p2_in[vi];
        `R_P3    : v = p3_in[vi];
        `R_SCON  : v = sfr_scon[vi];
        `R_TCON  : v = sfr_tcon[vi];
        default  : begin
          //Pini one bit output
          vu = {4'b0010, a[6:3]};
          //v  = iram(conv_integer(vu))[vi];
          r_ar = vu[6:0];
          v  = 
          (vi == 0) ? r_dr[0] :
          (vi == 1) ? r_dr[1] :
          (vi == 2) ? r_dr[2] :
          (vi == 3) ? r_dr[3] :
          (vi == 4) ? r_dr[4] :
          (vi == 5) ? r_dr[5] :
          (vi == 6) ? r_dr[6] : r_dr[7];
        end//        default  : begin 
      endcase//      case (vu) 
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  task SET_BIT;
    input  [7:0]  a;
    input         v;
    reg    [7:0] vu;
    integer      vi;
    begin
      r_wer = 1'b0;//Pini
      vu    = {a[7:3], 3'b000};
      vi    =  a[2:0];
      case (vu)
        `R_B     : sfr_b[vi]    = v;
        `R_ACC   : sfr_acc[vi]  = v;
        `R_PSW   : sfr_psw[vi]  = v;
        `R_IP    : sfr_ip[vi]   = v;
        `R_IE    : sfr_ie[vi]   = v;
        `R_SP    : sfr_sp[vi]   = v;
        `R_P0    : p0_out[vi]   = v;
        `R_P1    : p1_out[vi]   = v;
        `R_P2    : p2_out[vi]   = v;
        `R_P3    : p3_out[vi]   = v;
        `R_SCON  : sfr_scon[vi] = v;
        `R_TCON  : sfr_tcon[vi] = v;
        default : begin
          //Pini one bit output
          vu = {4'b0010, a[6:3]};
          //iram(conv_integer(vu))[vi] = v;
          r_dr[0]=(vi == 0) ? v :r_dr[0];
          r_dr[1]=(vi == 1) ? v :r_dr[1];
          r_dr[2]=(vi == 2) ? v :r_dr[2];
          r_dr[3]=(vi == 3) ? v :r_dr[3];
          r_dr[4]=(vi == 4) ? v :r_dr[4];
          r_dr[5]=(vi == 5) ? v :r_dr[5];
          r_dr[6]=(vi == 6) ? v :r_dr[6];
          r_dr[7]=(vi == 7) ? v :r_dr[7];
          r_ar   = vu[6:0]; 
          r_wer  = 1'b1;
        end//        default : begin 
      endcase        //      case (vu) 
    end//    begin 
  endtask
  
  //////////////////////////////////////////////////////////////////////////////-
  
  reg [7:0] v8;
  reg       v1;
  
  always @ (posedge rst or posedge clk) begin
    
    if( rst == 1'b1 ) begin
      sfr_b    = `C0_8;
      sfr_acc  = `C0_8;
      sfr_psw  = `C0_8;
      sfr_ie   = `C0_8;
      sfr_ip   = `C0_8;
      sfr_sp   = `CD_8;
      sfr_dpl  = `C0_8;
      sfr_dph  = `C0_8;
      sfr_pcon = `C0_8;
      sfr_scon = `C0_8;
      sfr_sbuf = `C0_8;
      sfr_tcon = `C0_8;
      sfr_tmod = `C0_8;
      sfr_tl0  = `C0_8;
      sfr_th0  = `C0_8;
      sfr_tl1  = `C0_8;
      sfr_th1  = `C0_8;
      
      out_data = `CD_8;
      out_bit_data = 1'b0;
      p0_out   = `CD_8;
      p1_out   = `CD_8;
      p2_out   = `CD_8;
      p3_out   = `CD_8;
      r_wer    = 1'b0; //Pini
      
    end//    if( rst == 1'b1 ) begin 
    else begin
      
      if( rd == 1'b1 ) begin
        if( is_bit_addr == 1'b1 ) begin
          GET_BIT(addr, v1);
          out_bit_data = v1;
        end//        if( is_bit_addr == 1'b1 ) begin 
        else begin
          GET_BYTE(addr, v8);
          out_data = v8;
        end//        else begin 
      end//      if( rd == 1'b1 ) begin 
      else begin
        if( wr == 1'b1 ) begin
          if( is_bit_addr == 1'b1 ) begin
            SET_BIT(addr, in_bit_data);
          end//            if( is_bit_addr == 1'b1 ) begin 
          else begin
            SET_BYTE(addr, in_data);
          end//            else begin 
        end//          if( wr == 1'b1 ) begin 
      end//        else  begin 
    end//    else begin 
  end//  begin 
endmodule// 

//////////////////////////////////////////////////////////////////////////////-

// end of file //// 


